namespace StudySystemClient {
	public class ActivitiesView : Gtk.Box {
		private Client client;
		private Gtk.FlowBox card_container;
		private RefreshingIndicator refreshing_indicator;

		public ActivitiesView(Client client)  {
			Object(orientation: Gtk.Orientation.VERTICAL,
				   hexpand: true,
				   vexpand: true,
				   margin_top: 0,
				   margin_bottom: 0,
				   margin_start: 0,
				   margin_end: 0);

			this.client = client;

			card_container = new Gtk.FlowBox();
			card_container.homogeneous = true;
			card_container.min_children_per_line = 1;
			card_container.max_children_per_line = 1;
			card_container.selection_mode = Gtk.SelectionMode.NONE;
			card_container.valign = Gtk.Align.START;

			var scrolled_window = new Gtk.ScrolledWindow();
			scrolled_window.hscrollbar_policy = Gtk.PolicyType.NEVER;
			scrolled_window.hexpand = true;
			scrolled_window.vexpand = true;
			scrolled_window.add_css_class("card-container");
			scrolled_window.set_child(card_container);

			var overlay = new Gtk.Overlay();
			overlay.hexpand = overlay.vexpand = true;
			overlay.set_child(scrolled_window);
			this.append(overlay);

			refreshing_indicator = new RefreshingIndicator(overlay);

			refresh.begin((obj, res) => {
					refresh.end(res);
				});
		}

		private async void refresh() {
			refreshing_indicator.show();
			try {
				var activities = yield client.list_activities();
				card_container.remove_all();
				foreach (var activity in activities) {
					var card = new ActivityCard(activity);
					card.session_logged.connect(log_session);
					card_container.append(card);
				}
			} catch (ClientError e) {
				stderr.printf("Error refreshing activities: %s\n",
							  e.message);
			}
			refreshing_indicator.hide();
		}

		private async void log_session(string subject, ActivityType type,
									   int minutes) {
			try {
				yield client.log_session(subject, type, minutes);
				yield refresh();
			} catch (ClientError e) {
				stderr.printf("Error logging session: %s\n", e.message);
			}
		}
	}

	private class RefreshingIndicator {
		private Gtk.Overlay overlay;
		private Gtk.Frame frame;

		public RefreshingIndicator(Gtk.Overlay overlay) {
			this.overlay = overlay;

			var label = new Gtk.Label("Refreshing");
			label.halign = Gtk.Align.START;

			var spinner = new Gtk.Spinner();
			spinner.start();

			var content = new Gtk.Box(Gtk.Orientation.HORIZONTAL, 6);
			content.margin_top = content.margin_bottom
				= content.margin_start = content.margin_end = 12;
			content.append(label);
			content.append(spinner);

			frame = new Gtk.Frame(null);
			frame.halign = Gtk.Align.CENTER;
			frame.valign = Gtk.Align.START;
			frame.add_css_class("osd");
			frame.set_child(content);
		}

		public void show() {
			overlay.add_overlay(frame);
		}

		public void hide() {
			overlay.remove_overlay(frame);
		}
	}
}
